'use client';
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { enrichEventWithDetails, useStylesheet, useSyncRef } from '@ui5/webcomponents-react-base';
import { clsx } from 'clsx';
import { cloneElement, forwardRef, isValidElement, useCallback, useMemo } from 'react';
import { Cell, Label as RechartsLabel, Legend, Pie, PieChart as PieChartLib, Sector, Text as RechartsText, Tooltip, } from 'recharts';
import { getValueByDataKey } from 'recharts/lib/util/ChartUtils.js';
import { useLegendItemClick } from '../../hooks/useLegendItemClick.js';
import { useOnClickInternal } from '../../hooks/useOnClickInternal.js';
import { ChartContainer } from '../../internal/ChartContainer.js';
import { defaultFormatter } from '../../internal/defaults.js';
import { tooltipContentStyle, tooltipFillOpacity } from '../../internal/staticProps.js';
import { classNames, styleData } from './PieChart.module.css.js';
import { PieChartPlaceholder } from './Placeholder.js';
const tooltipItemDefaultStyle = { color: 'var (--sapTextColor)' };
/**
 * A Pie Chart is a type of graph that displays data in a circular graph.
 * The pieces of the graph are proportional to the fraction of the whole in each category.
 *
 * In other words, each slice of the pie is relative to the size of that category in the group as a whole.
 * The entire “pie” represents 100 percent of a whole, while the pie “slices” represent portions of the whole.
 */
const PieChart = forwardRef((props, ref) => {
    const { loading, loadingDelay, dataset, noLegend, noAnimation, tooltipConfig, onDataPointClick, onLegendClick, onClick, centerLabel, style, className, slot, ChartPlaceholder, children, ...rest } = props;
    useStylesheet(styleData, PieChart.displayName);
    const [componentRef, chartRef] = useSyncRef(ref);
    const isDonutChart = props['data-component-name'] === 'DonutChart';
    const chartConfig = {
        margin: { right: 30, left: 30, bottom: 30, top: 30, ...(props.chartConfig?.margin ?? {}) },
        legendPosition: 'bottom',
        legendHorizontalAlign: 'center',
        paddingAngle: 0,
        outerRadius: '80%',
        resizeDebounce: 250,
        tooltipItemStyle: tooltipItemDefaultStyle,
        ...props.chartConfig,
    };
    const showActiveSegmentDataLabel = chartConfig.showActiveSegmentDataLabel ?? true;
    const dimension = useMemo(() => ({
        formatter: defaultFormatter,
        ...props.dimension,
    }), [props.dimension]);
    const measure = useMemo(() => ({
        formatter: defaultFormatter,
        ...props.measure,
    }), [props.measure]);
    const dataLabel = (props) => {
        const hideDataLabel = typeof measure.hideDataLabel === 'function' ? measure.hideDataLabel(props) : measure.hideDataLabel;
        if (hideDataLabel || chartConfig.activeSegment === props.index)
            return null;
        if (isValidElement(measure.DataLabel)) {
            return cloneElement(measure.DataLabel, { ...props, config: measure });
        }
        return (_jsx(RechartsText, { ...props, alignmentBaseline: "middle", className: "recharts-pie-label-text", children: measure.formatter(props.value) }));
    };
    const tooltipValueFormatter = useCallback((value, name) => [measure.formatter(value), dimension.formatter(name)], [measure.formatter, dimension.formatter]);
    const onItemLegendClick = useLegendItemClick(onLegendClick, () => measure.accessor);
    const onClickInternal = useOnClickInternal(onClick);
    const onDataPointClickInternal = useCallback((payload, dataIndex, event) => {
        if (payload && payload && typeof onDataPointClick === 'function') {
            onDataPointClick(enrichEventWithDetails(event, {
                value: payload.value,
                dataKey: payload.tooltipPayload?.[0]?.dataKey,
                name: payload.name,
                payload: payload.payload,
                dataIndex,
            }));
        }
    }, [onDataPointClick]);
    // REUSE: part of this function is copied from: https://github.com/recharts/recharts/blob/411e57a3c206a1425ff33a7e63cacf40a844e551/storybook/stories/Examples/Pie/CustomActiveShapePieChart.stories.tsx#L22-L44
    const renderActiveShape = useCallback((props) => {
        const RADIAN = Math.PI / 180;
        const { cx, cy, midAngle, innerRadius, outerRadius, startAngle, endAngle, fill, payload, percent, value } = props;
        const sin = Math.sin(-RADIAN * midAngle);
        const cos = Math.cos(-RADIAN * midAngle);
        const sx = cx + (outerRadius + 10) * cos;
        const sy = cy + (outerRadius + 10) * sin;
        const mx = cx + (outerRadius + 30) * cos;
        const my = cy + (outerRadius + 30) * sin;
        const ex = mx + (cos >= 0 ? 1 : -1) * 22;
        const ey = my;
        const textAnchor = cos >= 0 ? 'start' : 'end';
        const activeLegendItem = chartRef.current?.querySelector(`.legend-item-${chartConfig.activeSegment}`);
        if (activeLegendItem && !activeLegendItem?.dataset.activeLegend) {
            const allLegendItems = chartRef.current?.querySelectorAll('.recharts-legend-item');
            allLegendItems.forEach((item) => item.removeAttribute('data-active-legend'));
            activeLegendItem.setAttribute('data-active-legend', 'true');
        }
        return (_jsxs("g", { children: [isDonutChart && (_jsx("text", { x: cx, y: cy, dy: 8, textAnchor: "middle", fill: fill, children: payload.name })), _jsx(Sector, { cx: cx, cy: cy, innerRadius: innerRadius, outerRadius: outerRadius, startAngle: startAngle, endAngle: endAngle, fill: fill }), _jsx(Sector, { cx: cx, cy: cy, startAngle: startAngle, endAngle: endAngle, innerRadius: outerRadius + 6, outerRadius: outerRadius + 10, fill: fill }), showActiveSegmentDataLabel && (_jsxs(_Fragment, { children: [_jsx("path", { d: `M${sx},${sy}L${mx},${my}L${ex},${ey}`, stroke: fill, fill: "none" }), _jsx("circle", { cx: ex, cy: ey, r: 2, fill: fill, stroke: "none" }), _jsx("text", { x: ex + (cos >= 0 ? 1 : -1) * 12, y: ey, textAnchor: textAnchor, fill: fill, children: measure.formatter(value) }), _jsx("text", { x: ex + (cos >= 0 ? 1 : -1) * 12, y: ey, dy: 18, textAnchor: textAnchor, fill: fill, children: `(${(percent * 100).toFixed(2)}%)` })] }))] }));
    }, [showActiveSegmentDataLabel, chartConfig.activeSegment, isDonutChart]);
    const renderLabelLine = useCallback((props) => {
        const hideDataLabel = typeof measure.hideDataLabel === 'function' ? measure.hideDataLabel(props) : measure.hideDataLabel;
        if (hideDataLabel || chartConfig.activeSegment === props.index)
            return null;
        return Pie.renderLabelLineItem({}, props, undefined);
    }, [chartConfig.activeSegment, measure.hideDataLabel]);
    const legendWrapperStyle = useMemo(() => {
        if (chartConfig.activeSegment != null && showActiveSegmentDataLabel) {
            if (chartConfig.legendPosition === 'bottom') {
                return {
                    paddingBlockStart: '30px',
                };
            }
            else if (chartConfig.legendPosition === 'top') {
                return {
                    paddingBlockEnd: '30px',
                };
            }
        }
        return null;
    }, [showActiveSegmentDataLabel, chartConfig.activeSegment, chartConfig.legendPosition]);
    const { chartConfig: _0, dimension: _1, measure: _2, ...propsWithoutOmitted } = rest;
    return (_jsx(ChartContainer, { dataset: dataset, ref: componentRef, loading: loading, loadingDelay: loadingDelay, Placeholder: ChartPlaceholder ?? PieChartPlaceholder, style: style, className: className, slot: slot, resizeDebounce: chartConfig.resizeDebounce, ...propsWithoutOmitted, children: _jsxs(PieChartLib, { onClick: onClickInternal, margin: chartConfig.margin, accessibilityLayer: chartConfig.accessibilityLayer, className: clsx(typeof onDataPointClick === 'function' || typeof onClick === 'function' ? 'has-click-handler' : undefined, classNames.piechart), children: [_jsxs(Pie, { onClick: onDataPointClickInternal, innerRadius: chartConfig.innerRadius, outerRadius: chartConfig.outerRadius, paddingAngle: chartConfig.paddingAngle, nameKey: dimension.accessor, dataKey: measure.accessor, data: dataset, animationBegin: 0, isAnimationActive: !noAnimation, labelLine: renderLabelLine, label: dataLabel, activeIndex: chartConfig.activeSegment, activeShape: chartConfig.activeSegment != null && renderActiveShape, rootTabIndex: -1, children: [centerLabel && _jsx(RechartsLabel, { position: "center", children: centerLabel }), dataset &&
                            dataset.map((data, index) => (_jsx(Cell, { name: dimension.formatter(getValueByDataKey(data, dimension.accessor, '')), fill: measure.colors?.[index] ?? `var(--sapChart_OrderedColor_${(index % 12) + 1})` }, index)))] }), tooltipConfig?.active !== false && (_jsx(Tooltip, { cursor: tooltipFillOpacity, formatter: tooltipValueFormatter, contentStyle: tooltipContentStyle, itemStyle: chartConfig.tooltipItemStyle, labelStyle: chartConfig.tooltipLabelStyle, ...tooltipConfig })), !noLegend && (
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                _jsx(Legend, { verticalAlign: chartConfig.legendPosition, align: chartConfig.legendHorizontalAlign, onClick: onItemLegendClick, wrapperStyle: legendWrapperStyle, ...chartConfig.legendConfig })), children] }) }));
});
PieChart.displayName = 'PieChart';
export { PieChart };
